;///////////////////////////////////////////////////////////////////////////////////////////////////
;/										Tic-Tac-Toe					   							   /
;/								Written by Donald Straney 6/25/03								   /
;/						Designed for the Z80/TI-86 graphing calculator							   /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////DEFINITIONS/////////////////////////////////////////////////

#include "ti86asm.inc"

;CALLS

_divHLby10	equ		4044h				; hl = hl/10

;CONSTANTS

Pos_1		equ		$1030				;Variable-width cursor positions for board squares
Pos_2		equ		$1039
Pos_3		equ		$1043

Pos_4		equ		$1930
Pos_5		equ		$1939
Pos_6		equ		$1943

Pos_7		equ		$2230
Pos_8		equ		$2239
Pos_9		equ		$2243

Msg_Pos_1	equ		$2a2a				;Variable-width cursor positions for messages
Msg_Pos_2	equ		$301e
Msg_Pos_3	equ		$3633

Title_Pos_1	equ		$000a				;Variable-width cursor positions for title messages
Title_Pos_2	equ		$0615

Num_X		equ		1					;Numbers on virtual board representing pieces
Num_O		equ		2
Num_Blank	equ		0

Delay_Coeff	equ		65535				;Delay between win, new round

Board_Start_Pos	equ	$fc00+(16*16)+6	;Starting position in video memory for board graphic

;VARS

Board_Pos_1	equ		$ca00				;Virtual board-each represents a square on the board
Board_Pos_2	equ		$ca01				;a 0,1, or 2 is stored in each to represent blank, X, or O
Board_Pos_3	equ		$ca02
Board_Pos_4	equ		$ca03
Board_Pos_5	equ		$ca04
Board_Pos_6	equ		$ca05
Board_Pos_7	equ		$ca06
Board_Pos_8	equ		$ca07
Board_Pos_9	equ		$ca08

Go_First	equ		$ca09				;Sets which player goes first-0 is user, 1 is calc		

C_Wins		equ		$ca0a				;Number of calc's wins
Y_Wins		equ		$ca0b				;Number of user's wins
Ties		equ		$ca0c				;Number of ties

Pos_Flags	equ		$ca0d				;Second virtual board-1 bit/square
Pos_Flags_2	equ		$ca0e				;used to test for wins

Test_Num	equ		$ca0f				;Piece to test for on board-used by AI

_@RandomNumberSeed	equ	$ca10			;Random number seed

Save_Num	equ		$ca11

;/////////////////////////////////////////START OF PROGRAM//////////////////////////////////////////

.org _asm_exec_ram

New_Game:
	call _runindicoff					;Make sure that run indicator is off
	ld a,r								;Seed random number generator
	ld (_@RandomNumberSeed),a
	ld b,13								;Set to clear all vars
	jr Clear_Vars
New_Round:
	ld b,9								;Set to clear just virtual board-not scores
	ld a,(Go_First)						;Toggle goes-first var
	or a
	jr z,Set_Calc_First					;If user has moved first last round, calc moves first this time
Set_User_First:							;If calc has moved first last round, user moves first this time
	xor a
	jr Set_First_Player
Set_Calc_First:
	ld a,1
Set_First_Player:
	ld (Go_First),a

Clear_Vars:								;Set vars specified in b to 0
	ld hl,$ca00
	xor a
Clear_Vars_Loop:
	ld (hl),a
	inc hl
	djnz Clear_Vars_Loop

	call _clrScrn						;When done, clear screen and text shadow
	ld hl,Board_Pic						;Draw board on screen
	ld de,Board_Start_Pos
	ld bc,12
	ld a,26
Board_Draw_Loop:
	push bc
	ldi
	ldi
	ldi
	ldi
	pop bc
	ex de,hl
	add hl,bc
	ex de,hl
	dec a
	jr nz,Board_Draw_Loop

	ld hl,Title_Pos_1					;Set small cursor to top of screen
	ld (_penCol),hl
	ld hl,Title_Text_1					;and draw title text
	call _vputs
	ld hl,Title_Pos_2
	ld (_penCol),hl
	ld hl,Title_Text_2
	call _vputs
	ld hl,Msg_Pos_1						;Refresh scores-first set cursor position
	ld (_penCol),hl
	ld hl,Msg_Text_1					;then draw text
	call _vputs
	ld a,(Y_Wins)						;Get user's score
	call Draw_Score						;and draw
	ld hl,Msg_Pos_2						;Set cursor position for yet another message
	ld (_penCol),hl
	ld hl,Msg_Text_2					;then draw message text again
	call _vputs
	ld a,(C_Wins)						;Get calc's score
	call Draw_Score						;and draw
	ld hl,Msg_Pos_3						;Set cursor position for last message FINALLY
	ld (_penCol),hl
	ld hl,Msg_Text_3					;then draw LAST MESSAGE!!!
	call _vputs
	ld a,(Ties)							;Get number of ties
	call Draw_Score						;and draw!!!

	ld a,(Go_First)						;See which player is supposed to go first
	or a
	jp nz,AI_Decide						;If calc goes first, let it move

Key_Loop:								;Keypress loop
	call _getkey						;Wait for keypress
	sub $07								;If exit key,
	ret z								;then return
	sub $16								;If 1 key,
	jr z,Draw_X_7						;draw an x in lower-left
	dec a								;If 2 key,
	jr z,Draw_X_8						;draw an x in lower-middle
	dec a								;If 3 key,
	jr z,Draw_X_9						;draw an x in lower-right
	dec a								;If 4 key,
	jr z,Draw_X_4						;draw an x in center-left
	dec a								;If 5 key,
	jr z,Draw_X_5						;draw an x in center-middle
	dec a								;If 6 key,
	jr z,Draw_X_6						;draw an x in center-right
	dec a								;If 7 key,
	jr z,Draw_X_1						;draw an x in upper-left
	dec a								;If 8 key,
	jr z,Draw_X_2						;draw an x in upper-middle
	dec a								;If 9 key,
	jr z,Draw_X_3						;draw an x in upper-right
;	cp kClear							;If clear key,
;	jr z,Draw_Board						;then clear board (by redrawing)
	jr Key_Loop							;If none of these, wait for another keypress

Draw_X_1:
	ld hl,Pos_1							;Point to first position on board
	ld e,$00							;Point to virtual board byte
	jr Draw_X							;Now draw it
Draw_X_2:
	ld hl,Pos_2
	ld e,$01
	jr Draw_X
Draw_X_3:
	ld hl,Pos_3
	ld e,$02
	jr Draw_X
Draw_X_4:
	ld hl,Pos_4
	ld e,$03
	jr Draw_X
Draw_X_5:
	ld hl,Pos_5
	ld e,$04
	jr Draw_X
Draw_X_6:
	ld hl,Pos_6
	ld e,$05
	jr Draw_X
Draw_X_7:
	ld hl,Pos_7
	ld e,$06
	jr Draw_X
Draw_X_8:
	ld hl,Pos_8
	ld e,$07
	jr Draw_X
Draw_X_9:
	ld hl,Pos_9
	ld e,$08	
Draw_X:
	ld d,$ca						;Complete virtual board pointer
	ld a,(de)						;Look at position on virtual board
	or a							;Is it clear?
	jr nz,Key_Loop					;If it isn't, then don't put the X there-wait for another keypress
Draw_X_Now:							;If it is clear, draw an X
	ld a,Num_X
	ld (de),a
	ld b,l
	ld c,h
	ld hl,Sprite_X
	call PutSprite
	call Test_User_Win				;See if user has 3 Xs in a row
	jr nz,Check_Board_Full			;If user hasn't won, check if board is full, then AI will move
	ld hl,Y_Wins					;If user has won, increment score
	inc (hl)
	call Delay						;wait a second
	jp New_Round					;and start a new round
Check_Board_Full:
	call Test_Board_Full			;Test if board is full
	jr z,AI_Decide					;If it isn't, the AI will move
	ld hl,Ties						;If it is, increment the number of ties
	inc (hl)
	call Delay						;wait a second
	jp New_Round					;then start a new round

AI_Decide:							;Now the AI will decide how to move
	ld a,Num_O						;First, see if can make 3 in a row
	ld (Test_Num),a
	jr Test_Board_Put_O
AI_Try_Block:
	ld a,Num_X						;If not, then try to block user
	ld (Test_Num),a

Test_Board_Put_O:					;Place O to block or to win	
	ld de,Board_Pos_1				;Start at first board position
	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_1					;If not, skip
	inc de							;If it is, look at position 2
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 1,2, put an O on 3 to block
Skip_1:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_2					;If not, skip
	inc de							;If it is, look at position 3
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_2					;If it is, then Xs are on 1,3, put an O on 2 to block
Skip_2:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 2
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_3					;If not, skip
	inc de							;If it is, look at position 3
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 2,3, put an O on 1 to block
Skip_3:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_4					;If not, skip
	inc de							;If it is, look at position 4
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 1,4, put an O on 7 to block
Skip_4:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_5					;If not, skip
	ld de,Board_Pos_7				;If it is, look at position 7
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_4					;If it is, then Xs are on 1,7, put an O on 4 to block
Skip_5:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 4
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_6					;If not, skip
	inc de							;If it is, look at position 7
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 4,7, put an O on 1 to block
Skip_6:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_7					;If not, skip
	ld de,Board_Pos_5				;If it is, look at position 5
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 1,5, put an O on 9 to block
Skip_7:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_8					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 1,9, put an O on 5 to block
Skip_8:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld de,Board_Pos_5				;Point to space 5
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_9					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_1					;If it is, then Xs are on 5,9, put an O on 1 to block
Skip_9:
	ld de,(Save_Num)				;Restore position

	inc de							;Point to space 2

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_10					;If not, skip
	inc de							;If it is, look at position 8
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_2					;If it is, then Xs are on 5,8, put an O on 2 to block
Skip_10:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_11					;If not, skip
	inc de							;If it is, look at position 5
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_8					;If it is, then Xs are on 2,5, put an O on 8 to block
Skip_11:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_12					;If not, skip
	ld de,Board_Pos_8				;If it is, look at position 8
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 2,8, put an O on 5 to block
Skip_12:
	ld de,(Save_Num)				;Restore position

	inc de							;Point to space 3

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_13					;If not, skip
	inc de							;If it is, look at position 6
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 3,6, put an O on 9 to block
Skip_13:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_14					;If not, skip
	ld de,Board_Pos_9				;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_6					;If it is, then Xs are on 3,9, put an O on 6 to block
Skip_14:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 6
	inc de
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_15					;If not, skip
	inc de							;If it is, look at position 9
	inc de
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 6,9, put an O on 3 to block
Skip_15:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_16					;If not, skip
	inc de							;If it is, look at position 5
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 3,5, put an O on 7 to block
Skip_16:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_17					;If not, skip
	ld de,Board_Pos_7					;If it is, look at position 7
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 3,7, put an O on 5 to block
Skip_17:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	inc de
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_18					;If not, skip
	inc de							;If it is, look at position 7
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_3					;If it is, then Xs are on 5,7, put an O on 3 to block
Skip_18:
	ld de,(Save_Num)				;Restore position

	inc de							;Point to space 4

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_19					;If not, skip
	inc de							;If it is, look at position 5
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_6					;If it is, then Xs are on 4,5, put an O on 6 to block
Skip_19:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_20					;If not, skip
	inc de							;If it is, look at position 6
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_5					;If it is, then Xs are on 4,6, put an O on 5 to block
Skip_20:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 5
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_21					;If not, skip
	inc de							;If it is, look at position 6
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_4					;If it is, then Xs are on 5,6, put an O on 4 to block
Skip_21:
	ld de,(Save_Num)				;Restore position

	inc de							;Point to space 7
	inc de
	inc de

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_22					;If not, skip
	inc de							;If it is, look at position 8
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_9					;If it is, then Xs are on 7,8, put an O on 9 to block
Skip_22:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_23					;If not, skip
	inc de							;If it is, look at position 9
	inc de
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_8					;If it is, then Xs are on 7,9, put an O on 8 to block
Skip_23:
	ld de,(Save_Num)				;Restore position

	ld hl,Test_Num					;Get piece to test for
	ld (Save_Num),de				;Save position
	inc de							;Point to space 8
	ld a,(de)						;Get piece at that space
	cp (hl)							;Is piece that's being tested?
	jr nz,Skip_24					;If not, skip
	inc de							;If it is, look at position 9
	ld a,(de)						;Get piece on that square
	cp (hl)							;Is piece that's being tested?
	call z,Draw_O_7					;If it is, then Xs are on 8,9, put an O on 7 to block
Skip_24:
	ld de,(Save_Num)				;Restore position

	ld a,(Test_Num)					;Is it checking for wins?
	cp Num_O
	jp z,AI_Try_Block				;If it is, then hasn't found any possibilities, now try to block user

	jp Put_Random					;Otherwise, put on random square

Draw_O_1:
	ld c,$00						;Set pointer to virtual board position
	ld hl,Pos_1						;Set cursor position
	jr Put_O						;Check if space is empty, then draw
Draw_O_2:
	ld c,$01
	ld hl,Pos_2
	jr Put_O
Draw_O_3:
	ld c,$02
	ld hl,Pos_3
	jr Put_O
Draw_O_4:
	ld c,$03
	ld hl,Pos_4
	jr Put_O
Draw_O_5:
	ld c,$04
	ld hl,Pos_5
	jr Put_O
Draw_O_6:
	ld c,$05
	ld hl,Pos_6
	jr Put_O
Draw_O_7:
	ld c,$06
	ld hl,Pos_7
	jr Put_O
Draw_O_8:
	ld c,$07
	ld hl,Pos_8
	jr Put_O
Draw_O_9:
	ld c,$08
	ld hl,Pos_9
Put_O:
	push hl
	pop de
	pop hl								;Get address that was called from
	ld b,$ca							;Complete virtual board address
	ld a,(bc)							;Look at position on board where O would be put
	or a								;Is it empty?
	jr z,Put_O_Now						;If it is, put O on virtual board and screen
	jp (hl)								;Otherwise, return to calling address and check next space
Put_O_Now:
	ld a,Num_O							;Put O on virtual board
	ld (bc),a
	ld c,d
	ld b,e
	ld hl,Sprite_O
	call PutSprite
	call Test_Calc_Win					;Test if calc has won
	jr nz,Check_Board_Full_2			;If it hasn't, check if board is full, then wait for user to move
	ld hl,C_Wins						;If it has, increment calc's number of wins
	inc (hl)
	call Delay							;wait a second
	jp New_Round						;and start new round
Check_Board_Full_2:
	call Test_Board_Full				;Test if board is full
	jp z,Key_Loop						;If it isn't, the user will move
	ld hl,Ties							;If it is, increment the number of ties
	inc (hl)	
	call Delay							;wait a second
	jp New_Round						;then start a new round


Put_Random:
	call RandomNumber					;Generate a random number
	and $0f								;Get the lower 4 bits-must be 15 or under
	sub 9								;Is above 8?
	jr nc,Jump_To_Random				;If it isn't, go to space defined by r
	add a,9								;Otherwise, add 9 again first
Jump_To_Random:
	call z,Draw_O_1						;If 0, go to first space
	dec a
	call z,Draw_O_2						;If 1, go to second space
	dec a
	call z,Draw_O_3						;If 2, go to third space
	dec a
	call z,Draw_O_4						;If 3, go to fourth space
	dec a
	call z,Draw_O_5						;If 4, go to fifth space
	dec a
	call z,Draw_O_6						;If 5, go to sixth space
	dec a
	call z,Draw_O_7						;If 6, go to seventh space
	dec a
	call z,Draw_O_8						;If 7, go to eighth space
	dec a
	call z,Draw_O_9						;If 8, go to ninth space
	jr Put_Random						;If one is taken, then try again

Test_Board_Full:						;Reset z if board is full
	ld hl,Board_Pos_1					;Start at beginning of board
	ld b,9								;Board is 9 squares
Test_Board_Full_Loop:
	ld a,(hl)							;Look at square on board
	or a								;Is there anything there?
	ret z								;If there's a space without anything on it, signal that and return
	inc hl								;Otherwise, look at next space
	djnz Test_Board_Full_Loop			;and loop
	ret									;Only gets here if board is full-last square couldn't have
										;been empty, had to have been 1 or 2, so z is already reset

Test_User_Win:							;Test if user has won
	ld b,Num_X							;Set to recognize user's pieces
	jr Test_Win							;Now test
Test_Calc_Win:							;Test if calc has won
	ld b,Num_O							;Set to recognize calc's pieces
Test_Win:								;Set z if selected player has won has won
										;First, create 9-bit virtual board-each bit corresponds to one
										;square on real board, is set if X is on it (when testing for
										;user win) or if O is on it (testing for calc win), then masks
										;applied to test certain squares for patterns, to detect wins
	ld de,Pos_Flags						;Set pointer to byte w/flags
	ld hl,Board_Pos_1					;Start at beginning of virtual board
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_1						;If not, reset corresponding flag
	call z,Set_1						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_2						;If not, reset corresponding flag
	call z,Set_2						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_3						;If not, reset corresponding flag
	call z,Set_3						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_4						;If not, reset corresponding flag
	call z,Set_4						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_5						;If not, reset corresponding flag
	call z,Set_5						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_6						;If not, reset corresponding flag
	call z,Set_6						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_7						;If not, reset corresponding flag
	call z,Set_7						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_8						;If not, reset corresponding flag
	call z,Set_8						;If it is, set corresponding flag
	inc l								;Point to next board position
	ld a,(hl)							;Get square
	cp b								;Is it the piece of specified player?
	call nz,Reset_9						;If not, reset corresponding flag
	call z,Set_9						;If it is, set corresponding flag
										;Now test flag byte for winning combinations
	ld a,(de)							;Get flag byte 1
	ld b,a								;Save in b
	and %00000111						;Test first combination-mask out all irrelevant bits
	cp %0000111							;Now test
	jr z,End_Test_Positions				;If it matches, then end, z bit is set
	ld a,b								;Restore flag byte
	and %01001001
	cp %01001001
	jr z,End_Test_Positions
	ld a,b
	and %10010010
	cp %10010010
	jr z,End_Test_Positions
	ld a,b
	and %00111000
	cp %00111000
	jr z,End_Test_Positions
	ld a,b
	and %01010100
	cp %01010100
	jr z,End_Test_Positions
	ld a,b
	and %11000000
	cp %11000000
	call z,Test_Bit_9
	ld a,b
	and %00100100
	cp %00100100
	call z,Test_Bit_9
	ld a,b
	and %00010001
	cp %00010001
	call z,Test_Bit_9
	ld a,1								;No wins found-reset z bit
End_Test_Positions:
	pop hl								;Get called-from address
	jp (hl)								;Return to that

Test_Bit_9:
	pop hl
	ld a,(Pos_Flags_2)
	cp %00000001
	jr z,End_Test_Positions
	jp (hl)

Set_1:
	ld a,(de)
	set 0,a
	ld (de),a
	ret

Set_2:
	ld a,(de)
	set 1,a
	ld (de),a
	ret

Set_3:
	ld a,(de)
	set 2,a
	ld (de),a
	ret

Set_4:
	ld a,(de)
	set 3,a
	ld (de),a
	ret

Set_5:
	ld a,(de)
	set 4,a
	ld (de),a
	ret

Set_6:
	ld a,(de)
	set 5,a
	ld (de),a
	ret

Set_7:
	ld a,(de)
	set 6,a
	ld (de),a
	ret

Set_8:
	ld a,(de)
	set 7,a
	ld (de),a
	ret

Set_9:
	inc de
	ld a,(de)
	set 0,a
	ld (de),a
	dec de
	ret

Reset_1:
	ld a,(de)
	res 0,a
	ld (de),a
	ret

Reset_2:
	ld a,(de)
	res 1,a
	ld (de),a
	ret

Reset_3:
	ld a,(de)
	res 2,a
	ld (de),a
	ret

Reset_4:
	ld a,(de)
	res 3,a
	ld (de),a
	ret

Reset_5:
	ld a,(de)
	res 4,a
	ld (de),a
	ret

Reset_6:
	ld a,(de)
	res 5,a
	ld (de),a
	ret

Reset_7:
	ld a,(de)
	res 6,a
	ld (de),a
	ret

Reset_8:
	ld a,(de)
	res 7,a
	ld (de),a
	ret

Reset_9:
	inc de
	ld a,(de)
	res 0,a
	ld (de),a
	dec de
	ret

Delay:
	push hl
	pop de
	ld bc,Delay_Coeff
	ldir
	push hl
	pop de
	ld bc,Delay_Coeff
	ldir
	ret

Draw_Score:
	ld l,a								;Put a into hl
	ld h,0
										;Now go on to DisplayHL

;====================================================================
; DisplayHL:  (idea from SCaBBy/Mardell)    [Assembly Coder's Zenith]
;  Display HL as a 5-digit decimal number with no leading zeros
;  out: AF, HL, BC, DE destroyed
;====================================================================
DisplayHL:
 ld c,'0'  				; save ascii value for zero
 ld de,_OP1+5			; point to end of the buffer
 xor a				; zero terminate string
 ld (de),a				; set last byte to zero
DisplayHLl:
 call _divHLby10		; next digit
 dec de				; next position
 add a,c				; convert to ascii
 ld (de),a				; save digit
 ld a,h				; load upper byte
 or l					; check with lower byte for zero
 jr nz,DisplayHLl			; loop
 ex de,hl				; point to buffer
 jp _vputs			; print number, we're done

;===========================================
; RandomNumber     [Assembly Coder's Zenith]
; out: A = psuedo random number
; modified: AF, BC    (14 bytes/62 t-states)
;===========================================
RandomNumber:
  ld a,r
  ld b,a
  ld a,($1234)
  ld c,a
  ld a,(bc)
  ld (_@RandomNumberSeed),a
  xor b
  sub b
  add a,c
  ret

; =============================================================

; Puts a sprite stored at (HL) at B,C

; =============================================================

PutSprite:
 push bc                        ; Save BC (X, Y) 
 push de                        ; Save DE
 push hl                        ; Save HL (Start of Sprite Data)
 push hl                        ; Save HL (Start of Sprite Data)
 push bc
 ld d, c
 ld e, b
 call FindPixel                 ; Finds pixel at E, D
 pop bc
                                ; PutSprite needs pixel at B, C
                                ; A = Bitmask with one bit set; this will be the bit (pixel) "counter"
 ex de,hl                       ; DE = Address in Display of Sprite
 pop hl                         ; HL = Start of Sprite Data
 ld b,(hl)                      ; B = X width of Sprite 
 inc hl                         ; Next byte
 ld c,(hl)                      ; C = Y width of Sprite
 inc hl                         ; Next byte (now pointing at actual sprite data)
 push hl                        ; Save the twice incremented HL into stack
 pop ix                         ; IX = The recently twice incremented HL
 ex de,hl                       ; HL = Address in Display of Sprite
PS_NewRow:
 push bc                        ; Save X width of Sprite and Y width of Sprite
 ld d,(ix)                      ; D = (IX), so D = First byte from Sprite
 inc ix                         ; IX points to next row
 push af                        ; Save Bitmask 
 push hl                        ; Save HL, Address in Display of Sprite
PS_NewCol:                      ; Now the fun begins, remember A is the bitmask, D = the Left Column Byte from Sprite
 rl d                           ; Rotate D left through carry, Leftmost pixel of Sprite Byte is carry
 ld e,a                         ; E = bitmask
 jr nc,PS_NoPixel               ; Check pixel from sprite to see if it is set
 or (hl)                        ; Sets Pixel at the "current bit" (Bitmask is the "bit counter")
 ld (hl),a                      ; (HL) = A, Save changes to
 jr PS_NextPixel
PS_NoPixel:
 cpl                            ; Invert Bitmask
 and (hl)                       ; Clears pixel at the "current bit" (Bitmask is the "bit counter")
 ld (hl),a                      ; (HL) = A Save changes
PS_NextPixel:
 ld a,e                         ; A = Bitmask
 rrca                           ; A is rotated right *through* carry
 jr nc,PS_SameByte              ; If the carry was set, that means that one bit set in A (bit counter) 
                                ;  rotated all the way to the end right into carry and recycles back into Bit 7
                                ;  so it can be used for the next byte
 inc hl                         ; Go to the next byte in the display
PS_SameByte:
 djnz PS_NewCol                 ; B = X (width of sprite), so it loops to PS_NewCol X times. This means that is X = 6, 
                                ; it will Shift Right the bitmask (move the bit counter to the right) 6 times, comparing 
                                ; each bit of the bitmask and the sprite and setting or clearing the pixel in that
                                ; particular bit. It then moves on the the next pixel.
                                ; Move on to the next row
 pop hl                         ; Recover HL, the Address in Display of the Sprite
 pop af                         ; Recover AF, A = Bitmask
 ld de,16                       ; DE = 16
 add hl,de                      ; HL = HL + 16, this moves down ONE row in the Display Area (128 width / (8 bits/pixel) = 16 bytes)
 pop bc                         ; Recover X_width and Y_height of Sprite
 dec c                          ; C = Y_height of Sprite, subract one, which means one row of the Sprite has been drawn
 jr nz,PS_NewRow                ; If there are more rows to be drawn, go back to PS_NewRow
; No more rows. Since there were "effectively" 3 pushes before PS_NewRow, there must be three pops that way the
; ret statement will retrieve the correct address when it returns to the calling program.
 pop hl                                         
 pop de
 pop bc
 ret

;==============================================================

; Dan Eble & James Yopp FindPixel routine

; Input:  D = y, E = x

; Output: HL= addr in vid mem, A = bitmask, C is modified

; =============================================================

FindPixel:
	ld hl,FP_Bits
	ld a,e
	and $07         ; a = bit offset
	add a,l
	ld l,a
	adc a,h
	sub l
	ld h,a
	ld c,(hl)       ; c = bitmask for (hl)
;48 t-states up to this point
	ld hl,FP_RLD
	ld (hl),d
	ld a,e          ; a = x/8 (byte offset within row)
	rrca
	rrca   
	rrca
	rld
	or $FC 
	ld l,(hl)
	ld h,a          ; hl -> byte in vid mem
	ld a,c          ; now a = bitmask for (hl)
;121 t-states up to this point
	ret

FP_RLD:  .db $00

FP_Bits: .db $80,$40,$20,$10,$08,$04,$02,$01

;////////////////////////////////////////////STRINGS////////////////////////////////////////////////

Sprite_X:
	.db 8, 8
	.db %00000000
	.db %01000100
	.db %00101000
	.db %00010000
	.db %00101000
	.db %01000100
	.db %00000000
	.db %00000000

Sprite_O:
	.db 8, 8
	.db %00011000
	.db %00100100
	.db %01000010
	.db %01000010
	.db %01000010
	.db %00100100
	.db %00011000
	.db %00000000

Board_Pic:
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %11111111, %11111111, %11111111, %11000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %11111111, %11111111, %11111111, %11000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000
	.db %00000000, %10000000, %01000000, %00000000

Msg_Text_1:
	.db "Your wins: ",0
Msg_Text_2:
	.db "Calculator's wins:  ",0
Msg_Text_3:
	.db "Ties: ",0

Title_Text_1:
	.db "Tic-Tac-Toe by Donald Straney",0
Title_Text_2:
	.db "straney@umbi.umd.edu",0

.end